/***************************************************************************
                          icecc.cpp  -  description
                             -------------------
    begin                :   14 21:00:28 EEST 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>

#include <qaccel.h>
#include <qmessagebox.h>

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpixmap.h>
#include <qtooltip.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "icecc.h"
#include "qcentralwidget.h"
#include "tools.h"
#include "parsecfg.h"

#include "iceccimg.xpm"
#include "icontoolbar.xpm"
#include "exit.xpm"

#include "qthemebrowser.h"

IceCCApp::IceCCApp():
themeBrowser(0),
icewmCfgDir(getenv("ICEWM_PRIVCFG")),
aTools(COUNTTOOLS),
themeDir(PREFIX),
cmdIndexHelp("icecchelp ")
{
    setCaption(tr("IceWM Control Center " VERSION));

    initMenuBar();
    initStatusBar();

    cWidget = new QCentralWidget(aTools, this);
    setCentralWidget(cWidget);

    setMaximumSize(cWidget->width(),
                   cWidget->height() + menuBar()->height() + statusBar()->height());

    if(icewmCfgDir.isNull())
        icewmCfgDir = QDir::homeDirPath() + "/.icewm";

	QString optFileName(icewmCfgDir);
	int res;
	char *bufValue;
	
    optFileName += "/.icecc";
	/*
	 * Get dir of theme and current theme
	 */
	themeDir += "/share/icecc/themes/";
	res = fetchVarFromCfgFile((const char*)optFileName.local8Bit(),
				"ThemeDir", &bufValue, CFG_STRING, CFG_SIMPLE, 0, NULL);
	if(res == 0)
	{
		themeDir = bufValue;
		free(bufValue);
	} /* 	if(res == 0) */
	res = fetchVarFromCfgFile((const char*)optFileName.local8Bit(),
				"CurrentTheme", &bufValue, CFG_STRING, CFG_SIMPLE, 0, NULL);
	if(res == 0)
	{
		themeName = bufValue;
		free(bufValue);
	} /* 	if(res == 0) */
	if(themeName.isEmpty())
		loadBuiltinTheme();
	else
		loadTheme();
	setTheme();

	cmdIndexHelp += PREFIX;
	cmdIndexHelp += "/share/icecc/help/default/index.html";
}//constructor

IceCCApp::~IceCCApp()
{
}//destructor


const char *toolAccel[] = {"Ctrl+M", "Ctrl+P", "Ctrl+W", "Ctrl+T",
                           "Ctrl+K", "Ctrl+S", "Ctrl+H", "Ctrl+B",
			   "Ctrl+I", "Ctrl+U", "Ctrl+E"};


void IceCCApp::initMenuBar()
{

  toolMenu = new QPopupMenu(this);
  toolSubMenu = new QPopupMenu(this);
  toolMenu->insertItem(QPixmap(icontoolbar_xpm),tr("&Tools"), toolSubMenu);

  toolMenu->insertSeparator();
  toolMenu->insertItem(tr("T&hemes..."), this, SLOT(slotBrowseTheme()), 
		  CTRL+Key_T);

  toolMenu->insertSeparator();
  toolMenu->insertItem(QPixmap(exit_xpm),tr("&Quit"), qApp, SLOT(quit())
		  , CTRL+Key_Q);

  helpMenu=new QPopupMenu();
  helpMenu->insertItem(tr("&Index"), this, SLOT(slotHelpIndex()));
  helpMenu->insertSeparator();
  helpMenu->insertItem(tr("&About"), this, SLOT(slotHelpAbout()));

  menuBar()->insertItem(tr("&Tools"), toolMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);
}/* initMenuBar() */

void IceCCApp::initStatusBar()
{
  statusBar()->message(tr("Ready."), 2000);
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void IceCCApp::slotHelpAbout()
{
	QString aboutStr(tr("IceWM Control Center\nVersion " VERSION "\n"
				"(c) 2002-2004 by Vadim A. Khohlov\n\nTheme: "));
	aboutStr += themeName.isEmpty() ? QString(tr("BuiltIn")) : themeName;
	if(!themeAuthor.isEmpty())
	{
		aboutStr += "\nAuthor: " + themeAuthor;
		if(!themeDescr.isEmpty())
			aboutStr += tr("\nDescription of theme: ") + themeDescr;
	} /* 	if(!themeAuthor.isEmpty()) */
  	QMessageBox::about(this,tr("About..."),aboutStr);
}

/**
 * Runs a tool with number numTool.
 */
void IceCCApp::runTool(int numTool)
{
    QString cmd(aTools[numTool]->command);
    cmd += '&';
    system(cmd.data());
}/* IceCCApp::runTools(int numTool) */

char *themeDirPtr;
char *currentThemePtr;
cfgStruct cfg[] = {
				{ "ThemeDir", CFG_STRING, &themeDirPtr},
				{ "CurrentTheme", CFG_STRING, &currentThemePtr}
					  };

/** No descriptions */
void IceCCApp::saveOptions()
{
	QString optFileName(icewmCfgDir);
    optFileName += "/.icecc";
	
	cfgStoreValue(cfg, "ThemeDir", (const char*)themeDir.local8Bit(), CFG_SIMPLE, 0);
	cfgStoreValue(cfg, "CurrentTheme", (const char*)themeName.local8Bit(), CFG_SIMPLE, 0);
	cfgDump((const char*)optFileName.local8Bit(), cfg, CFG_SIMPLE, 0);
}/* IceCCApp::saveOptions() */

/**
*/
void
IceCCApp::loadBuiltinTheme()
{
	for(unsigned i = 0; i < COUNTTOOLS; i++)
	{
		aTools.insert(i, new IToolInfo(
					QRect(rectCoords[i][0],
					   	  rectCoords[i][1],
						  rectCoords[i][2] - rectCoords[i][0],
						  rectCoords[i][3] - rectCoords[i][1]),
					tools[i], toolNames[i],
					toolAccel[i]));
	} /* 	for(unsigned i = 0; i < COUNTTOOLS; i++) */
	imageFileName.truncate(0);
} /* IceCCApp ::loadBuiltinTheme() */

/**
*/
void
IceCCApp::setTheme()
{
	QPixmap bgPixmap;
    if(imageFileName.isEmpty())
		bgPixmap = QPixmap(iceccimg_xpm);
	else
		bgPixmap = QPixmap(imageFileName);
    cWidget->setBackgroundPixmap(bgPixmap);
    cWidget->setMinimumSize(bgPixmap.size());
    cWidget->setMaximumSize(bgPixmap.size());
    setMaximumSize(cWidget->width(),
            cWidget->height() + menuBar()->height() + statusBar()->height());
    resize(cWidget->width(),
            cWidget->height() + menuBar()->height() + statusBar()->height());

	toolSubMenu->clear();
	for(uint i = 0; i <aTools.count(); i++)
	{
		  toolSubMenu->insertItem(aTools[i]->menuString, this,
	  		SLOT(runTool(int)), QAccel::stringToKey(aTools[i]->hotKey), i);
		  QToolTip::add(cWidget, aTools[i]->rc, aTools[i]->toolTip);
	} /*for(uint i = 0; i <aTools.count(); i++) */
} /* IceCCApp ::setTheme() */

/** Runs a dialog for browsing theme */
void IceCCApp::slotBrowseTheme()
{
    if(!themeBrowser)
    {
        themeBrowser = new QThemeBrowser(&themeDir, &themeName, this);
        CHECK_PTR(themeBrowser);
    }
    if( themeBrowser->exec() == QDialog::Accepted)
	{
		if(!themeName.isEmpty())
		{
			char *bufValue;
			int res;
			QString themeD(themeDir);
			themeD += themeName + "/";
			QString themeFile(themeD);
		
			themeFile 	+= "theme";
			res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
					"image", &bufValue, CFG_STRING, CFG_INI, 0, "General");
			if(!res)
			{
				imageFileName = themeD + bufValue;
				free(bufValue);
			} /* if(!res) */
			loadTheme();
		} /* if(!themeName.isEmpty()) */
		else
		{
			imageFileName = "";
			loadBuiltinTheme();
		} /* 		else */
		saveOptions();
		setTheme();
	} /* if( themeBrowser->exec() == QDialog::Accepted) */
} /* void IceCCApp::slotBrowseTheme() */

/**
*/
void
IceCCApp::loadTheme()
{
	uint i;
	int res;
	uint qtools;
	char toolName[10];
	char *bufValue;
	//const char *file;
	int left = 0, top = 0, height = 0, width = 0;
	char *command, *menuCommand, *hotKey, *toolTip, *nullStr = NULL;
	
	QString themeFile(themeDir);
	themeFile += themeName + "/theme";

	//file = (const char*)themeFile.local8Bit();
	//debug("%s", file);
	//debug("%s", themeFile.latin1());
	//return;
	res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(), "qtools", &qtools, 
			CFG_UINT, CFG_INI, 0, "General");
	if(res < 0)
	{
		QMessageBox::critical(this, tr("Error!"), 
				QString(tr("The quantity of tools is not specified.\n"
					"The BuiltIn theme will be loaded.")));
		loadBuiltinTheme();
		return;
	} /* 		if(res < 0) */
	res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(), "image", &bufValue, 
			CFG_STRING, CFG_INI, 0, "General");
	if(res < 0)
	{
		QMessageBox::critical(this, tr("Error!"), 
				QString(tr("The background image is not specified.\n"
					"The BuiltIn theme will be loaded.")));
		loadBuiltinTheme();
		return;
	} /* 		if(res < 0) */
	imageFileName = themeDir + themeName + '/' + bufValue;
	free(bufValue);
	res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(), "author", &bufValue, 
			CFG_STRING, CFG_INI, 0, "General");
	if(res >= 0)
	{
		themeAuthor = bufValue;
		free(bufValue);
	} /* 	if(res >= 0) */
	else
		themeAuthor = "";
	res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(), "description", &bufValue, 
			CFG_STRING, CFG_INI, 0, "General");
	if(res >= 0)
	{
		themeDescr = bufValue;
		free(bufValue);
	} /* 	if(res >= 0) */	
	else
		themeDescr = "";
	/* Remove old tooltips.	 */	
	for(i = 0; i < aTools.count(); i++)
		QToolTip::remove(cWidget, aTools[i]->rc);
	/* Remove old tools. */
	aTools.clear();
	aTools.resize(qtools);
	uint ctools = 0;
	for(i = 0; i < qtools; i++)
	{
		sprintf(toolName, "Tool%d", i);
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"left", &left, CFG_INT, CFG_INI, 0, toolName);
		if(res < 0)
		{
			QMessageBox::critical(this, tr("Error!"), 
					QString(tr("For tool #%1 parametr \'left\' is not specified.")).arg(i));
			continue;
		} /* 		if(res < 0) */
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"top", &top, CFG_INT, CFG_INI, 0, toolName);
		if(res < 0)
		{
			QMessageBox::critical(this, tr("Error!"), 
					QString(tr("For tool #%1 parametr \'top\' is not specified.")).arg(i));
			continue;
		} /* 		if(res < 0) */
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"width", &width, CFG_INT, CFG_INI, 0, toolName);
		if(res < 0)
		{
			QMessageBox::critical(this, tr("Error!"), 
					QString(tr("For tool #%1 parametr \'width\' is not specified.")).arg(i));
			continue;
		} /* 		if(res < 0) */
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"height", &height, CFG_INT, CFG_INI, 0, toolName);
		if(res < 0)
		{
			QMessageBox::critical(this, tr("Error!"), 
					QString(tr("For tool #%1 parametr \'height\' is not specified.")).arg(i));
			continue;
		} /* 		if(res < 0) */
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"command", &command, CFG_STRING, CFG_INI, 0, toolName);
		if(res < 0)
		{
			QMessageBox::critical(this, tr("Error!"), 
					QString(tr("For tool #%1 parametr \'command\' is not specified.")).arg(i));
			continue;
		} /* 		if(res < 0) */
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"menuCommand", &menuCommand, CFG_STRING, CFG_INI, 0, toolName);
		if(res < 0)
		{
			QMessageBox::critical(this, tr("Error!"), 
					QString(tr("For tool #%1 parametr \'menuCommand\'"
					   " is not specified.")).arg(i));
			continue;
		} /* 		if(res < 0) */
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"hotKey", &hotKey, CFG_STRING, CFG_INI, 0, toolName);
		if(res < 0)
			hotKey = nullStr;
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
		 		"toolTip", &toolTip, CFG_STRING, CFG_INI, 0, toolName);
		if(res < 0)
			toolTip = nullStr;
		 
		aTools.insert(ctools++, new IToolInfo(QRect(left, top, width, height),
					command, menuCommand, hotKey, toolTip));
		free(command);
		free(menuCommand);
		free(hotKey);
		free(toolTip);
	} /* 	for(i = 0; i < qtools; i++) */
} /* IceCCApp ::loadTheme() */

/**
* runs a help system.
*/
void
IceCCApp::slotHelpIndex()
{
	system((const char*)cmdIndexHelp.local8Bit());
} /* IceCCApp ::slotHelpIndex() */
