/***************************************************************************
                          qthemebrowser.cpp  -  description
                             -------------------
    begin                : ??? ??? 26 2003
    copyright            : (C) 2003 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include <qdir.h>
#include <qfileinfo.h>

#include <stdlib.h>

#include "qthemebrowser.h"
#include "parsecfg.h"

extern const char *iceccimg_xpm[];

QThemeBrowser::QThemeBrowser(QString *tD, QString *tN,
							QWidget *parent, const char *name ):
QDialog(parent,name),
numCurrentTheme(0),
themeDir(tD),
themeName(tN)
{
	int w1, w2;
    setCaption(tr("Themes' browser"));

    QVBoxLayout *topLayout = new QVBoxLayout(this, 0);
    CHECK_PTR(topLayout);
	
	QHBox *hb = new QHBox(this);
	CHECK_PTR(hb);
	topLayout->addWidget(hb);

	//list of themes
	QVBox *vb = new QVBox(hb);
	CHECK_PTR(vb);
	QLabel *lb = new QLabel(tr("&Themes:"), vb);
	CHECK_PTR(lb);
	lbThemes = new QListBox(vb);
	CHECK_PTR(lbThemes);
	lb->setBuddy(lbThemes);

	//preview label
	lPreview = new QLabel("P", hb);
	CHECK_PTR(lPreview);
	lPreview->setScaledContents(true);

	//info about theme
	hb = new QHBox(this);
	CHECK_PTR(hb);
	topLayout->addWidget(hb);
	vb = new QVBox(hb);
	CHECK_PTR(vb);
	lb = new QLabel(tr(" Author:"), vb);
	CHECK_PTR(lb);
	lb->setFrameStyle(QFrame::Panel | QFrame::Raised);
	w1 = lb->width();
	QLabel *lb1 = new QLabel(tr(" Description:"), vb);
	CHECK_PTR(lb1);
	lb1->setFrameStyle(QFrame::Panel | QFrame::Raised);
	w2 = lb1->width();
	if (w2 > w1)
		w1 = w2;
	lb->setMaximumWidth(w1);
	lb1->setMaximumWidth(w1);
	vb->setMaximumWidth(w1);
	vb = new QVBox(hb);
	CHECK_PTR(vb);
	lbAuthor = new QLabel("a", vb);
	CHECK_PTR(lbAuthor);
	lbAuthor->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	lbDescription = new QLabel("d", vb);
	CHECK_PTR(lbDescription);
	lbDescription->setFrameStyle(QFrame::Panel | QFrame::Sunken);

	//editor of themes' dir
	hb = new QHBox(this);
	CHECK_PTR(hb);
	topLayout->addWidget(hb);
	lb = new QLabel(tr("T&hemes' dir:"), hb);
	CHECK_PTR(lb);
	edThemeDir = new QLineEdit(*themeDir, hb);
	CHECK_PTR(edThemeDir);
	lb->setBuddy(edThemeDir);
	pbBrowse = new QPushButton(tr("&Browse..."), hb);
	CHECK_PTR(pbBrowse);
	QPushButton *pbReload = new QPushButton(tr("&Reload themes..."), hb);
	CHECK_PTR(pbReload);

	//ok, cancel buttons	
	hb = new QHBox(this);
	CHECK_PTR(hb);
	topLayout->addWidget(hb);
	QPushButton *ok = new QPushButton("Ok", hb);
	CHECK_PTR(ok);
	ok->setDefault(true);
	QPushButton *cancel = new QPushButton("Cancel", hb);
	CHECK_PTR(cancel);

    connect(lbThemes, SIGNAL(highlighted(int)), SLOT(slotHighLighted(int)));
    connect(cancel, SIGNAL(clicked()), SLOT(reject()));
    connect(ok, SIGNAL(clicked()), SLOT(slotOk()));
    connect(pbReload, SIGNAL(clicked()), SLOT(loadThemes()));
    connect(pbBrowse, SIGNAL(clicked()), SLOT(slotBrowse()));
	
	loadThemes();
	lbThemes->setCurrentItem(numCurrentTheme);
}//constructor

QThemeBrowser::~QThemeBrowser()
{
}//destructor

/**
* Loads list of themes.
*/
void
QThemeBrowser::loadThemes()
{
	lbThemes->clear();
	/*
	 * This function may be called when user will press the Reload button.
	 * So we must update themeDir.
	 */
	QString dirName(edThemeDir->text());
	QDir tDir(dirName);
	lbThemes->insertItem(tr("Builtin theme"));

	tDir.setFilter(QDir::Dirs);
	const QFileInfoList *list = tDir.entryInfoList();
	if(!list)
	{
		QMessageBox::critical(this, tr("Error!"),
				tr("Can't read %1!").arg(dirName));
		return;
	} /* 	if(!list) */
	*themeDir = dirName;
	QFileInfoListIterator it(*list);
	QFileInfo *fi;

	while((fi = it.current()))
	{
		/*
		 * Add directory with theme file and not hidden
		 */
		 if(fi->fileName()[0] != '.' && 
				QFile::exists(fi->absFilePath() + "/theme"))
			lbThemes->insertItem(fi->fileName());
		++it;
	} /* 	while((fi = it.current())) */
} /* QThemeBrowser ::loadThemes() */

void
QThemeBrowser::slotHighLighted(int index)
{
	if(index == 0)	
	{//this is a built-in theme
		lPreview->setPixmap(QPixmap(iceccimg_xpm));
		lbAuthor->setText("Vadim A. Khohlov");
		lbDescription->setText("Builtin theme");
	} /* 	if(index == 0)	 */
	else
	{
		char *bufValue;
		int res;
		QString themeD(*themeDir);
		themeD += "/" + lbThemes->text(index);
		QString themeFile(themeD);
		QString imageFile(themeD);
		imageFile += '/';
		themeFile 	+= "/theme";
		
		QFileInfo fi(themeFile);
		if(!fi.isReadable())
		{
			QMessageBox::critical(this, tr("Error!"),
				tr("Can't read %1!").arg(themeFile));
			return;
		} /* 		if(!fi.isReadable()) */
		
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"image", &bufValue, CFG_STRING, CFG_INI, 0, "General");
		imageFile += bufValue;
		free(bufValue);
		QPixmap pxm(imageFile);
		if(pxm.isNull())
			QMessageBox::critical(this, tr("Error!"),
				tr("Can't read image %1!").arg(imageFile));
		else
			lPreview->setPixmap(pxm);

		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"Author", &bufValue, CFG_STRING, CFG_INI, 0, "General");
		if(!res)
		{
			lbAuthor->setText(bufValue);
			free(bufValue);
		} /* 		if(!res) */
		else
			lbAuthor->setText(tr("Unknown"));
		
		res = fetchVarFromCfgFile((const char*)themeFile.local8Bit(),
				"Description", &bufValue, CFG_STRING, CFG_INI, 0, "General");
		if(!res)
		{
			lbDescription->setText(bufValue);
			free(bufValue);
		} /* 		if(!res) */
		else
			lbDescription->setText(tr("Unknown"));
	} /* 	else */
} /* QThemeBrowser ::slotHighlighted(int index) */

void
QThemeBrowser::slotOk()
{
	//*themeName = lbThemes->currentItem() ? lbThemes->currentText() : "";
	*themeName = lbThemes->currentItem() ? lbThemes->currentText() : QString::null;
	accept();
} /* QThemeBrowser ::slotOk() */

void
QThemeBrowser::slotBrowse()
{
	QString td = QFileDialog::getExistingDirectory(*themeDir,
			this, "ged", tr("Choose a directory"));
	if(!td.isEmpty())
	{
		edThemeDir->setText(td);
		loadThemes();
	} /* 	if(!td.isEmpty()) */
} /* QThemeBrowser ::slotBrowse() */
